provider_path = Pathname.new(__FILE__).parent.parent
require File.join(provider_path, 'vcenter')
require 'rbvmomi'
require 'resolv'
require 'net/ssh'

Puppet::Type.type(:esx_mem).provide(:default, :parent => Puppet::Provider::Vcenter) do
  @doc = "Configure and install MEM on ESX server"
  def configure_mem
    return false
  end

  def configure_mem=(value)
    flag = 0
    error_log_filename = "/tmp/configuremem_err_log.#{Process.pid}"
    log_filename = "/tmp/configuremem_log.#{Process.pid}"
    script_executable_path = resource[:script_executable_path]
    setup_script_filepath = resource[:setup_script_filepath]
    host_username = resource[:host_username]
    host_password = get_host_password 
    host_ip = resource[:name ]
    vnics = resource[:vnics]
    vnics_ipaddress = resource[:vnics_ipaddress]
    iscsi_vswitch = resource[:iscsi_vswitch]
    mtu = resource[:mtu]
    iscsi_vmkernal_prefix = resource[:iscsi_vmkernal_prefix]
    iscsi_netmask = resource[:iscsi_netmask]
    storage_groupip = resource[:storage_groupip]
    iscsi_chapuser = resource[:iscsi_chapuser]
    iscsi_chapsecret = get_iscsi_chapsecret
    if validate_configure_param.eql?(1)
      return 1
    end
    begin
      enterMaintenanceMode
      chap_extension = ""
      if !iscsi_chapuser.nil?
        chap_extension = "--chapuser #{iscsi_chapuser} --chapsecret #{iscsi_chapsecret}"
      end
      if resource[:disable_hw_iscsi].to_s.eql?('true')
        cmd = "perl #{script_executable_path}/#{setup_script_filepath} --configure --username #{host_username} --password #{host_password} --server=#{host_ip} --nics #{vnics} --ips #{vnics_ipaddress} --vswitch #{iscsi_vswitch} --mtu #{mtu} --vmkernel #{iscsi_vmkernal_prefix} --netmask #{iscsi_netmask} --groupip #{storage_groupip} #{chap_extension} --enableswiscsi --nohwiscsi"
      else
        cmd =  "perl #{script_executable_path}/#{setup_script_filepath} --configure --username #{host_username} --password #{host_password} --server=#{host_ip} --nics #{vnics} --ips #{vnics_ipaddress} --vswitch #{iscsi_vswitch} --mtu #{mtu} --vmkernel #{iscsi_vmkernal_prefix} --netmask #{iscsi_netmask} --groupip #{storage_groupip} #{chap_extension}"
      end
      flag = execute_system_cmd(cmd , log_filename , error_log_filename)
    rescue Exception => e
      Puppet.err "Unable to configure MEM on the server '#{name}' because the following exception occurred: -\n #{e.message}"
    end
    exitMaintenanceMode

    Puppet.notice "Successfully configured MEM on the server '#{name}'."
    return flag
  end

  def install_mem
    mem
  end

  # Intalling MEM
  def install_mem=(value)
    enterMaintenanceMode
    cmd = "perl #{resource[:script_executable_path]}/#{resource[:setup_script_filepath]}  --install --username #{resource[:host_username]} --password #{get_host_password } --server=#{resource[:name ]}"

    error_log_filename = "/tmp/installmem_err_log.#{Process.pid}"
    log_filename = "/tmp/installmem_log.#{Process.pid}"

    flag = execute_system_cmd(cmd , log_filename , error_log_filename)

    if flag.eql?(0)
      exitMaintenanceMode
      # restart hostd rather than reboot the entire machine
      toggle_ssh
      restart_hostd
      reset_ssh
    end
    return flag
  end

  def restart_hostd
    Puppet.info('Restarting hostd...')
    Net::SSH.start(resource[:name], resource[:host_username], :password=>get_host_password,
                   :paranoid => Net::SSH::Verifiers::Null.new, :global_known_hosts_file=>"/dev/null") do |ssh|
      ssh.exec('/etc/init.d/hostd restart')
    end
    #Need to ensure webclient and APIs are reachable, so we wait a bit, and then retry a call that will fail if they aren't completely running yet.
    sleep 15
    attempts = 1
    begin
      host(resource[:name]).RetrieveHardwareUptime
    rescue => e
      raise('ESXi Host remote management was not available after restarting hostd.') if attempts > 6
      Puppet.debug("ESXi host did not respond after restarting hostd.  Retrying in 15 seconds...")
      attempts += 1
      sleep 15
      retry
    end
    Puppet.info('hostd has been restarted successfully')
  end

  def get_esx_currentstate

    flag = 0
    host_ip = resource[:name]

    error_log_filename = "/tmp/err_#{host_ip}_state.#{Process.pid}"
    log_filename = "/tmp/log.#{host_ip}_state.#{Process.pid}"
    ENV['PERL_LWP_SSL_VERIFY_HOSTNAME']= '0' ;
    cmd = "vicfg-hostops --server #{host_ip} --username #{resource[:host_username]} --password #{get_host_password} --operation info"
    system(cmd , :out => [log_filename, 'a'], :err => [error_log_filename, 'a'])
    if $? != 0
      flag = 1
      err_content = File.open(error_log_filename, 'rb') { |file| file.read }
      Puppet.err err_content
    else
      content = File.open(log_filename, 'rb') { |file| file.read }
      if (/SOAP|Error|operation is not allowed/.match(content))
        # got some error
        Puppet.err content
        flag = 1
      else
        maintenance_status = /In Maintenance Mode\s+:\s+(\S+)/.match(content)
        if (maintenance_status[1].downcase.eql?('yes'))
          flag = 2
        else
          flag = 3
        end
      end

    end
    remove_files(log_filename, error_log_filename)
    return flag
  end

  def enterMaintenanceMode
    host(resource[:name]).EnterMaintenanceMode_Task(:timeout => 600,
                                   :evacuatePoweredOffVms => true).wait_for_completion
  rescue Exception => e
    fail "Could not enter maintenance mode because the following exception occured: -\n #{e.message}"
  end

  def exitMaintenanceMode
    host(resource[:name]).ExitMaintenanceMode_Task(:timeout => 600).wait_for_completion
  rescue Exception => e
    fail "Could not exit maintenance mode because the following exception occured: -\n #{e.message}"
  end

  private

  # Removing files
  def remove_files(logfile , errorfile)
    if File.exist?(logfile)
      File.delete(logfile)
    end

    if File.exist?(errorfile)
      File.delete(errorfile)
    end
  end

  # Check whether mem is installed on the ESX host
  def mem
    flag = 0

    cmd = "perl #{resource[:script_executable_path]}/#{resource[:setup_script_filepath]} --query --username #{resource[:host_username]} --password #{get_host_password } --server=#{resource[:name ]}"

    error_log_filename = "/tmp/err_log.#{Process.pid}"
    log_filename = "/tmp/log.#{Process.pid}"
    ENV['PERL_LWP_SSL_VERIFY_HOSTNAME']= '0' ;
    system(cmd , :out => [log_filename, 'a'], :err => [error_log_filename, 'a'])
    if $? != 0
      flag = 1
      err_content = File.open(error_log_filename, 'rb') { |file| file.read }
      Puppet.err err_content
    else
      content = File.open(log_filename, 'rb') { |file| file.read }
      if (/(?i:installed)/.match(content))
        Puppet.notice content
      else
        flag = 1
        Puppet.notice content
      end
    end

    remove_files( error_log_filename , log_filename)

    if flag.eql?(0)
      Puppet.info "MEM is already installed on the server."
      return "true"

    else
      Puppet.notice "MEM is not installed on the server."
      return "false"

    end
  end

  def execute_system_cmd(cmd,log_filename,error_log_filename)
    flag = 0
    ENV['PERL_LWP_SSL_VERIFY_HOSTNAME']= '0' ;
    system(cmd , :out => [log_filename, 'a'], :err => [error_log_filename, 'a'])
    if $? != 0
      flag = 1
      err_content = File.open(error_log_filename, 'rb') { |file| file.read }
      Puppet.err err_content
    else
      content = File.open(log_filename, 'rb') { |file| file.read }
      if (/(?i:Error|failed|SOAP|Enter maintenance mode|Could not find requested datastore|called at|pm line|does not exist|Could not bind)/.match(content))
        # got some error
        Puppet.err content
        flag = 1
      else
        Puppet.notice content
      end
    end
    remove_files(log_filename, error_log_filename)

    return flag

  end

  def validate_configure_param
    flag = 1
    if resource[:storage_groupip].nil?
      Puppet.err "Unable to configure MEM, because the 'storage_groupip' value is not provided."
    elsif resource[:mtu].nil?
      Puppet.err "Unable to configure MEM, because the 'mtu' value is not provided."
    elsif resource[:iscsi_vmkernal_prefix].nil?
      Puppet.err "Unable to configure MEM, because the 'iscsi_vmkernal_prefix' value is not provided."
    elsif resource[:iscsi_netmask].nil?
      Puppet.err "Unable to configure MEM, because the 'iscsi_netmask' value is not provided."
    elsif resource[:iscsi_vswitch].nil?
      Puppet.err "Unable to configure MEM, because the 'iscsi_vswitch' value is not provided."
    elsif resource[:vnics_ipaddress].nil?
      Puppet.err "Unable to configure MEM, because the 'vnics_ipaddress' value is not provided."
    elsif resource[:vnics].nil?
      Puppet.err "Unable to configure MEM, because the 'vnics' value is not provided."
    elsif !resource[:iscsi_chapuser].nil? and get_iscsi_chapsecret.nil?
      Puppet.err "Unable to configure MEM, because the 'iscsi_chapsecret' value is not provided."
    else
      flag = 0

    end
    return flag
  end
  
  def get_host_password
    resource[:host_password]
  end

  def get_iscsi_chapsecret
    resource[:iscsi_chapsecret]
  end

  # Turn TSM-SSH on if it is off
  def toggle_ssh
    if !ssh_state
      host(resource[:name]).configManager.serviceSystem.StartService(:id=>'TSM-SSH')
    end
  end

  # Let's turn TSM-SSH off it we had to turn it on
  def reset_ssh
    if !ssh_state
      host(resource[:name]).configManager.serviceSystem.StopService(:id=>'TSM-SSH')
    end
  end

  def ssh_state
    @ssh_state ||= ssh_svc.running
  end

  def ssh_svc
    @ssh_svc ||= 
        host(resource[:name]).config.service.service.find{|x| x.key == "TSM-SSH"} ||
        fail("service TSM-SSH not found")
  end

end
